/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function () {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.users.ManageUsers");

dojo.require("dojox.data.dom");

dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("com.ibm.team.repository.web.ui.internal.admin.ItemList");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.users.ImportUsersDialog");

var getAuthenticatedContributor = com.ibm.team.repository.web.client.session.getAuthenticatedContributor;
var AdminClient = com.ibm.team.repository.web.client.internal.AdminClient;
var ImportUsersDialog = com.ibm.team.repository.web.ui.internal.admin.users.ImportUsersDialog;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;

function setText(e, text){
	e.appendChild(document.createTextNode(text));
}

var ACTION_PREFIX = "com.ibm.team.repository.editUser&itemId=";
var SEARCH_ACTION = "action=com.ibm.team.repository.searchUsers";
var PAGE_SIZE = 50;

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.users.ManageUsers", com.ibm.team.repository.web.ui.internal.admin.ItemList, {
	
	postCreate: function() {
		setText(this.heading, "Active Users");
		setText(this.description, "Use this page to create and edit repository accounts.  To create a new account, click the " +
				"\"Create User\" button below.  To edit an account, click on an account in the list below. To search for" +
				" a particular user, enter a portion of the name, user ID, or e-mail address in the text-box below and click the Search button.");
		this.tableHeading.parentNode.removeChild(this.tableHeading);
		this.tableControls.style.paddingTop = "0.5em";
		this.searchText = document.createElement("input");
		this.searchText.type = "text";
		this.tableControls.appendChild(this.searchText);
		this.connect(this.searchText,"onkeypress",function (e) {
			if (e.keyCode == 13) {
				document.location.href = "#" + SEARCH_ACTION + "&searchTerm=" + encodeURIComponent(this.searchText.value);
			}
		});
		this.searchButton = document.createElement("button");
		this.searchButton.style.marginLeft = "5px";
		this.searchButton.appendChild(document.createTextNode("Search"));
		this.tableControls.appendChild(this.searchButton);
		this.connect(this.searchButton,"onclick",function (e) {
			dojo.stopEvent(e);
			document.location.href = "#" + SEARCH_ACTION + "&searchTerm=" + encodeURIComponent(this.searchText.value);
		});
		
		this.setHeaderRow(["Name", "User ID", "E-Mail"]);
		this.headerRow.cells[0].sortBy = "name";
		this.headerRow.cells[1].sortBy = "userId";
		this.headerRow.cells[2].sortBy = "emailaddress";
		dojo.toggleClass(this.headerRow, "sortable", true);
		this.connect(this.headerRow, "onclick", "updateSorting");
		this.connect(this.headerRow, "onmouseover", function (e) {
			dojo.toggleClass(e.target, "selected", true);
		});
		this.connect(this.headerRow, "onmouseout", function (e) {
			dojo.toggleClass(e.target, "selected", false);
		});
		
		// store the connect handle so we can disconnect later on if necessary
		this.newUserButtonCH = this.addNewButton(ACTION_PREFIX + "new", "Create User");
		this.newUserButton = this.newUserButtonCH[0];
		
		this.importUserButtonCH = this.addNewButton("","Import Users");
		this.importUserButton = this.importUserButtonCH[0];
		this.importUserButton.disabled = true;
		dojo.disconnect(this.importUserButtonCH);
		this.connect(this.importUserButton, "onclick", function (e) {
			var dialog = new ImportUsersDialog();
			this.connect(dialog,"onOk","performImport");
		});
		
		var srh = new ServiceResponseHandler(this, "getUserRegistryInfoCallback", "handleError");
		AdminClient.getUserRegistryInfo(srh);
		
		this.pageNumberContainer = document.createElement("div");
		this.pageNumberContainer.style.marginTop = "5px";
		this.pageNumberContainer.style.fontWeight = "normal";
		this.pageNumberContainer.style.display = "none";
		this.pageNumberContainer.appendChild(document.createTextNode("(Showing first " + PAGE_SIZE + " users)"));
		this.tableControls.appendChild(this.pageNumberContainer);
		
		this.refresh();
	},
	
	refresh: function () {
		this.clearState();
		var tr = document.createElement("tr");
		var td = document.createElement("td");
		td.appendChild(document.createTextNode("Searching for users..."));
		tr.appendChild(td);
		this.tbody.appendChild(tr);
	},
	
	show: function () {
		this.refresh();
	},
	
	reload: function () {
		this.refresh();
		this.runLatestSearch();
	},
	
	clearState: function () {
		this.searchText.value = "";
		dojox.data.dom.removeChildren(this.tbody);
		this.pageNumberContainer.style.display = "none";
		dojox.data.dom.removeChildren(this.messageBox);
		this.messageBox.className = "";
	},
	
	getUserRegistryInfoCallback: function (userRegistryInfo) {
		if (userRegistryInfo != null) {
			if (userRegistryInfo.queryable) {
				this.importUserButton.disabled = false;
			}
		}
	},
	
	performImport: function (users) {
		if (users != null && users.length > 0) {
			for (var i=0; i < users.length; i++) {
				if (users[i].emailAddress == null || users[i].emailAddress == "") {
					users[i].emailAddress = "unknown";
				}
			}
			var srh = new ServiceResponseHandler(this, "handleImportCallback", "errorImportCallback");
			AdminClient.postMultipleNewContributors({jsonUserInfo: dojo.toJson(users)}, srh);
		}
	},
	
	handleImportCallback: function (response) {
		this.runUserSearch(this.latestSearchTerm || "");
		if (response == null) {
			this.displaySuccessMessage("Users imported successfully");
		}
		else {
			var errObject = dojo.fromJson(response);
			var errNode = document.createElement("span");
			errNode.appendChild(document.createTextNode("Problems were encountered importing some of the users:"));
			errNode.appendChild(document.createElement("br"));
			for (var i=0; i < errObject.length; i++) {
				errNode.appendChild(document.createElement("br"));
				errNode.appendChild(document.createTextNode(errObject[i].userId + ": " + errObject[i].exception));
			}
			this.displayErrorMessage(errNode, true);
		}
	},
	
	errorImportCallback: function (err) {
		this.refresh();
	},
	
	runLatestSearch: function () {
		if (this.latestSearch != null) {
			if (this.latestSearch.type === "user") {
				this.searchText.value = this.latestSearch.searchTerm;
				this.runUserSearch(this.latestSearch.searchTerm, this.latestSearch.pageNum);
			}
			else if (this.latestSearch.type === "license") {
				this.runLicenseSearch(this.latestSearch.licenseName, this.latestSearch.pageNum);
			}
			else {
				this.runUserSearch("");
			}
		}
		else {
			this.runUserSearch("");
		}
	},
	
	runUserSearch: function (searchTerm, pageNum, sortBy, sortOrder) {
		if (searchTerm == null) {
			searchTerm = "";
		}
		if (sortBy == null) {
			sortBy = "name";
		}
		if (sortOrder !== "desc") {
			sortOrder = "asc";
		}
		if (pageNum == null) {
			pageNum = 0;
		}
		this.latestSearch = {
			type: "user",
			searchTerm: searchTerm,
			pageNum: pageNum,
			sortBy: sortBy,
			sortOrder: sortOrder
		};
		searchTerm = searchTerm.replace(/\*/g,"%");
		this.latestSearchTerm = searchTerm;
		this.latestSearchType = "searchTerm";
		this.queryPage = pageNum || 0;
		this.updatePagerDivLinks();
		dojox.data.dom.removeChildren(this.tbody);
		var tr = document.createElement("tr");
		var td = document.createElement("td");
		td.appendChild(document.createTextNode("Searching for users..."));
		tr.appendChild(td);
		this.tbody.appendChild(tr);
		var srh = new ServiceResponseHandler(this, "showUsers", "handleError");
		AdminClient.getContributors({searchTerm: searchTerm, pageSize: PAGE_SIZE, sortBy: sortBy, sortOrder: sortOrder, hideAdmin: true, hideGuest: true, hideUnassigned: true, pageNum: pageNum, hideArchivedUsers: true}, srh);
	},
	
	updateSorting: function (e) {
		if (this.latestSearch && this.latestSearch.type === "user") {
			var sortByField = e.target.sortBy;
			var sortOrder;
			if (this.latestSearch.sortBy === sortByField) {
				if (this.latestSearch.sortOrder === "desc") {
					sortOrder = "asc";
				}
				else {
					sortOrder = "desc";
				}
			}
			else {
				sortOrder = "asc";
			}
			document.location.href = "#" + SEARCH_ACTION + "&searchTerm=" + encodeURIComponent(this.latestSearch.searchTerm) + "&pageNum=" +
					this.latestSearch.pageNum + "&sortBy=" + sortByField + "&sortOrder=" + sortOrder;
		}
	},
	
	runLicenseSearch: function (licenseName, pageNum) {
		this.latestSearch = {
			type: "license",
			licenseName: licenseName,
			pageNum: pageNum
		};
		if (licenseName == null) {
			licenseName = "";
		}
		this.latestSearchTerm = licenseName;
		this.latestSearchType = "licenseType";
		this.queryPage = pageNum || 0;
		this.updatePagerDivLinks();
		dojox.data.dom.removeChildren(this.tbody);
		var tr = document.createElement("tr");
		var td = document.createElement("td");
		td.appendChild(document.createTextNode("Searching for users..."));
		tr.appendChild(td);
		this.tbody.appendChild(tr);
		var srh = new ServiceResponseHandler(this, "showUsers", "handleError");
		AdminClient.getContributors({searchTerm: licenseName, isLicenseSearch: true, pageSize: PAGE_SIZE, hideAdmin: true, hideGuest: true, hideUnassigned: true, pageNum: pageNum}, srh);
	},
	
	showUsers: function (resultSet) {
		dojox.data.dom.removeChildren(this.tbody);
		if (this.pagerDiv == null) {
			this.createPagerDiv();
		}
		if (resultSet == null || resultSet.elements == null || resultSet.elements.length == 0) {
			var tr = document.createElement("tr");
			var td = document.createElement("td");
			td.appendChild(document.createTextNode("No matching users found"));
			td.colSpan = 3;
			tr.appendChild(td);
			tr.onclick = function (e) {
				dojo.stopEvent(e);
			}
			this.tbody.appendChild(tr);
			dojo.toggleClass(this.pagerDiv.prevSpan,"disabled", true);
			dojo.toggleClass(this.pagerDiv.nextSpan,"disabled", true);
			dojox.data.dom.removeChildren(this.pagerDiv.rangeSpan);
			dojox.data.dom.removeChildren(this.pagerDiv.totalSpan);
			this.pagerDiv.rangeSpan.appendChild(document.createTextNode("0"));
			this.pagerDiv.totalSpan.appendChild(document.createTextNode("0"));
		}
		else {
			dojox.data.dom.removeChildren(this.pagerDiv.rangeSpan);
			dojox.data.dom.removeChildren(this.pagerDiv.totalSpan);
			if (resultSet.count > PAGE_SIZE) {
				var lowerBound = (this.queryPage * PAGE_SIZE) + 1;
				var upperBound = (this.queryPage + 1) * PAGE_SIZE;
				if (this.queryPage == 0) {
					dojo.toggleClass(this.pagerDiv.prevSpan,"disabled", true);
				}
				else {
					dojo.toggleClass(this.pagerDiv.prevSpan,"disabled", false);
				}
				if (upperBound >= resultSet.count) {
					upperBound = resultSet.count;
					dojo.toggleClass(this.pagerDiv.nextSpan,"disabled", true);
				}
				else {
					dojo.toggleClass(this.pagerDiv.nextSpan,"disabled", false);
				}
				this.pagerDiv.rangeSpan.appendChild(document.createTextNode(lowerBound + " - " + upperBound));
			}
			else {
				dojo.toggleClass(this.pagerDiv.prevSpan,"disabled", true);
				dojo.toggleClass(this.pagerDiv.nextSpan,"disabled", true);
				this.pagerDiv.rangeSpan.appendChild(document.createTextNode("1 - " + resultSet.count));
			}
			this.pagerDiv.totalSpan.appendChild(document.createTextNode(resultSet.count));
			var loggedInUser = getAuthenticatedContributor();
			dojo.forEach(resultSet.elements, dojo.hitch(this, function(user){
				var row = this.addRow(ACTION_PREFIX + user.itemId, [user.name, user.userId, user.emailAddress]);
				if (user.userId !== loggedInUser.userId) {
					var archiveIcon = document.createElement("img");
					archiveIcon.src = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/images/archive.gif");
					archiveIcon.title = "Archive User";
					archiveIcon.className = "archiveIcon";
					archiveIcon.userItemId = user.itemId;
					this.connect(archiveIcon,"onmouseover", function (e) {
						e.target.src = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/images/archive_hov.gif");
					});
					this.connect(archiveIcon, "onmouseout", function (e) {
						e.target.src = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/images/archive.gif");
					});
					this.connect(archiveIcon, "onclick", function (e) {
						dojo.stopEvent(e);
						this.archiveUser(e.target.userItemId);
					});
					row.cells[0].insertBefore(archiveIcon, row.cells[0].firstChild);
				}
				dojo.toggleClass(row.parentNode,"userTable",true);
			}));
		}
	},
	
	archiveUser: function (itemId) {
		var args = {
			"itemId": itemId,
			"archived": true
		}
		var srh = new ServiceResponseHandler(this, "archiveCallback", "archiveErrorCallback");
		AdminClient.postArchiveUser(args, srh);
	},
	
	archiveCallback: function (retVal) {
		var data = dojo.fromJson(retVal);
		if (data.errMessage) {
			this.displayErrorMessage("Error archiving user: " + data.errMessage);
		}
		else {
			this.reload();
		}
	},
	
	archiveErrorCallback: function (err) {
		this.displayErrorMessage("Error archiving user: " + err);
	},
	
	createPagerDiv: function () {
		this.pagerDiv = document.createElement("div");
		dojo.toggleClass(this.pagerDiv, "pager", true);
		var prevSpan = document.createElement("span");
		var prevLink = document.createElement("a");
		prevLink.appendChild(document.createTextNode("Previous"));
		prevSpan.appendChild(prevLink);
		var disabledPrev = document.createElement("span");
		disabledPrev.appendChild(document.createTextNode("Previous"));
		dojo.toggleClass(disabledPrev, "disabledText", true);
		prevSpan.appendChild(disabledPrev);
		this.pagerDiv.appendChild(prevSpan);
		this.pagerDiv.prevSpan = prevSpan;
		this.pagerDiv.prevLink = prevLink;
		this.pagerDiv.appendChild(document.createTextNode(" | "));
		var rangeSpan = document.createElement("span");
		dojo.toggleClass(rangeSpan, "bold", true);
		this.pagerDiv.appendChild(rangeSpan);
		this.pagerDiv.rangeSpan = rangeSpan;
		this.pagerDiv.appendChild(document.createTextNode(" of "));
		var totalSpan = document.createElement("span");
		dojo.toggleClass(totalSpan, "bold", true);
		this.pagerDiv.appendChild(totalSpan);
		this.pagerDiv.totalSpan = totalSpan;
		this.pagerDiv.appendChild(document.createTextNode(" | "));
		var nextSpan = document.createElement("span");
		var nextLink = document.createElement("a");
		nextLink.appendChild(document.createTextNode("Next"));
		nextSpan.appendChild(nextLink);
		var disabledNext = document.createElement("span");
		disabledNext.appendChild(document.createTextNode("Next"));
		dojo.toggleClass(disabledNext, "disabledText", true);
		nextSpan.appendChild(disabledNext);
		this.pagerDiv.appendChild(nextSpan);
		this.pagerDiv.nextSpan = nextSpan;
		this.pagerDiv.nextLink = nextLink;
		this.tableControls.appendChild(this.pagerDiv);
		this.updatePagerDivLinks();
		this.connect(prevLink,"onclick",function (e) {
			dojo.stopEvent(e);
			document.location.href = e.target.href;
		});
		this.connect(nextLink,"onclick",function (e) {
			dojo.stopEvent(e);
			document.location.href = e.target.href;
		});
	},
	
	updatePagerDivLinks: function () {
		if (this.pagerDiv != null) {
			this.pagerDiv.prevLink.href = "#" + SEARCH_ACTION + "&" + this.latestSearchType + "=" + encodeURIComponent(this.latestSearchTerm) + "&" + "pageNum=" + (this.queryPage - 1);
			this.pagerDiv.nextLink.href = "#" + SEARCH_ACTION + "&" + this.latestSearchType + "=" + encodeURIComponent(this.latestSearchTerm) + "&" + "pageNum=" + (this.queryPage + 1);
		}
	}
});

})();